/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse;

import static org.fortiss.tooling.kernel.utils.EcoreUtils.pickFirstInstanceOf;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.activation.UnsupportedDataTypeException;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.create.ArchitectureExplorationCreator;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.create.partitionmapping.PartitionMappingCreator;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.create.platform.PlatformCommunicationGraphCreator;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.create.safetyfunctionarch.SafetyFunctionArchCreator;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.create.taskmapping.AbstractTaskMappingCreator;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.create.taskmapping.TaskMappingCreator;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.Partition;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.platform.PlatformCommunicationGraphEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.safetyfunctionarch.SafetyFunctionArchEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.ComposableDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.graph.DecoderDependencyGraph;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.problem.CompositeProblemModuleBase;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.ArchitectureDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.ArchitectureExplorationProblemModule;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.StrictTTDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.comm.MessageDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.instantiatetaskgraph.InstantiatedTaskMappingDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.instantiatetaskgraph.InstantiatedTaskMappingDecoderAcyclic;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.partitionmapping.PartitionMappingDecoderGraph;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.partitionmapping.PartitionMappingIdentityDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.safetyfunction.SFEncodingDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.safetyfunction.SFGraphDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.safetyfunction.SFMappingConstraintDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.safetyfunction.SFMappingDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.taskmapping.AbstractTaskMappingIdentityDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.taskmapping.instantiate.AbstractTaskMappingDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.taskmapping.instantiate.TaskInstanceResourceAlignmentDecoder;
import org.fortiss.af3.exploration.alg.dse.evaluator.ArchitectureEvaluator;
import org.fortiss.af3.exploration.alg.dse.sysmodel.FailSilentExecModelFactory;
import org.fortiss.af3.exploration.alg.dse.sysmodel.FaultDetectionVotingExecModelFactory;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IGatewayUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IMemoryUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITransmissionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.moea.model.DseSpecification;
import org.fortiss.af3.exploration.moea.model.feature.SafetyExploration;
import org.fortiss.af3.exploration.moea.model.parameters.ExecutionModel;
import org.jgrapht.experimental.dag.DirectedAcyclicGraph.CycleFoundException;

/**
 * Factory to create exploration problems solved by the DSE. It is used to create the single parts
 * of the chain that define and solve the associated optimization problem.
 * 
 * @author diewald
 */
public class DSEFactory {
	/**
	 * References the currently valid {@link SystemModelAdapter} relevant for the current DSE run.
	 */
	private SystemModelAdapter<?, ?, ?, ?, ?, ?, ?> systemModelAdapter;

	/** The instance (singleton) of the {@link DSEFactory}. */
	private static final class InstanceHolder {
		/** the "real" instance. */
		static final DSEFactory INSTANCE = new DSEFactory();
	}

	/** Private constructor to prevent instantiation from external (singleton). */
	private DSEFactory() {
	}

	/** Returns the instance of this {@link DSEFactory}. */
	public static DSEFactory getInstance() {
		return InstanceHolder.INSTANCE;
	}

	/** Updates this Factory with a corresponding {@link SystemModelAdapter} of a DSE run. */
	// FIXME: Remove this method: it violates the sigleton pattern.
	public void init(SystemModelAdapter<?, ?, ?, ?, ?, ?, ?> systemModelAdapter) {
		this.systemModelAdapter = systemModelAdapter;
	}

	/**
	 * Creates/Registers the sub problem definitions that compose the overall architectural
	 * exploration problem.
	 * 
	 * @param archExpProblemModule
	 *            The {@link CompositeProblemModuleBase} declaring the architectural exploration
	 *            problem.
	 * @param dse
	 *            The design space exploration description.
	 * @param execDepGraph
	 *            Dependency graph defining the execution order.
	 * @throws Exception
	 *             if (one of) the sub problem(s) cannot be registered.
	 */
	@SuppressWarnings("unchecked")
	public
			<C, S extends InstantiatedTaskMappingEntry, T extends InstantiatedTaskMappingEncoding<S>>
			void createSubProblems(ArchitectureExplorationProblemModule archExpProblemModule,
					DseSpecification dse, DecoderDependencyGraph execDepGraph) throws Exception {
		// Java-limitation: it is not possible to pass the generic to the static methods.
		TaskMappingFactory<S, T> tmFactory =
				(TaskMappingFactory<S, T>)TaskMappingFactory.getInstance();
		SafetyExploration sfExplorationModule =
				pickFirstInstanceOf(SafetyExploration.class, dse.getFeatures());
		if(sfExplorationModule != null) {
			// Initialize & register safety function adapters with the SystemModelAdapter.
			sfExplorationModule.getAdapters().parallelStream()
					.forEach(a -> a.initializeAdapter(systemModelAdapter));
			systemModelAdapter.addSafetyFunctionAdapters(sfExplorationModule.getAdapters());
			createSafetyFunctionArchProblem(archExpProblemModule, execDepGraph);
		}
		createPartitionMappingProblem(archExpProblemModule, dse, execDepGraph);
		createPlatformCommunicationGraphProblem(archExpProblemModule, execDepGraph);
		createAbstractTaskMappingProblem(tmFactory, archExpProblemModule, dse, execDepGraph);
		createTaskMappingInstantiationProblem(tmFactory, archExpProblemModule, dse, execDepGraph);
	}

	/**
	 * Creates the safety function architecture exploration sub-problem, or, more specifically,
	 * the creators of this sub-problem.
	 *
	 * @param archExpProblemModule
	 *            The {@link CompositeProblemModuleBase} declaring the architectural exploration
	 *            problem.
	 * @param execDepGraph
	 *            Dependency graph defining the execution order.
	 * @throws Exception
	 *             if (one of) the sub problem(s) cannot be registered.
	 */
	@SuppressWarnings("unchecked")
	private <C, T extends SafetyFunctionArchEncoding<C>> void createSafetyFunctionArchProblem(
			ArchitectureExplorationProblemModule archExpProblemModule,
			DecoderDependencyGraph execDepGraph) throws Exception {
		// These suppress warnings are required since type erasure in Java does not allow to pass
		// XY<Z>.class.
		@SuppressWarnings("rawtypes") SafetyFunctionArchCreator sfCreator =
				new SafetyFunctionArchCreator(systemModelAdapter.getSafetyFunctionAdapters(),
						execDepGraph);
		@SuppressWarnings("rawtypes") SFEncodingDecoder sfEncodingDecoder =
				new SFEncodingDecoder<>();
		@SuppressWarnings("rawtypes") SFGraphDecoder sfGraphDecoder =
				new SFGraphDecoder(systemModelAdapter.getTaskGraphs());
		@SuppressWarnings("rawtypes") SFMappingDecoder sfMappingDecoder = new SFMappingDecoder();
		@SuppressWarnings("rawtypes") SFMappingConstraintDecoder sfMappingConstraintDecoder =
				new SFMappingConstraintDecoder();

		sfCreator.registerAssocDecoders(sfEncodingDecoder);
		sfCreator.registerAssocDecoders(sfGraphDecoder);
		sfCreator.registerAssocDecoders(sfMappingDecoder);
		sfCreator.registerAssocDecoders(sfMappingConstraintDecoder);

		@SuppressWarnings("rawtypes") List<ComposableDecoder> sfDecoders = new ArrayList<>();
		sfDecoders.add(sfEncodingDecoder);
		sfDecoders.add(sfGraphDecoder);
		sfDecoders.add(sfMappingDecoder);
		sfDecoders.add(sfMappingConstraintDecoder);

		execDepGraph.addCreatedEncoding(sfCreator.getCreatedEncodingType());
		archExpProblemModule.registerSubProblem(SafetyFunctionArchEncoding.class, sfCreator,
				sfDecoders);
	}

	/**
	 * Creates a Partition Mapping sub-problem: Partition --> ExecutionUnit.
	 * 
	 * @param archExpProblemModule
	 *            The {@link CompositeProblemModuleBase} declaring the architectural exploration
	 *            problem.
	 * @param dse
	 *            The design space exploration description.
	 * @param execDepGraph
	 *            Dependency graph defining the execution order.
	 * @throws CycleFoundException
	 *             if a cyclic dependency is discovered when registering the problem.
	 */
	@SuppressWarnings("unchecked")
	private void createPartitionMappingProblem(
			ArchitectureExplorationProblemModule archExpProblemModule, DseSpecification dse,
			DecoderDependencyGraph execDepGraph) throws CycleFoundException {
		Class<PartitionMappingEncoding> pmEnodingClass = PartitionMappingEncoding.class;
		PartitionMappingCreator pmCreator =
				new PartitionMappingCreator(execDepGraph,
						(Collection<IExecutionUnitAdapter<?>>)(Collection<?>)systemModelAdapter
								.getDeploymentTargets(), systemModelAdapter
								.getDeployableComponents().size());

		@SuppressWarnings("rawtypes") List<ComposableDecoder> pmDecoders = new ArrayList<>();
		PartitionMappingIdentityDecoder pmDecoder = new PartitionMappingIdentityDecoder();
		TaskInstanceResourceAlignmentDecoder tiraDecoder =
				new TaskInstanceResourceAlignmentDecoder();
		pmDecoders.add(pmDecoder);
		pmDecoders.add(tiraDecoder);
		pmCreator.registerAssocDecoders(pmDecoder);
		execDepGraph.addCreatedEncoding(pmCreator.getCreatedEncodingType());
		archExpProblemModule.registerSubProblem(pmEnodingClass, pmCreator, pmDecoders);
	}

	/**
	 * Creates a subproblem to obtain a flat graph of the communication network of target platform.
	 * It is intended to connect the virtual and physical architecture. The implementation as an
	 * encoding allows to consider changes made to the target platform during the exploration.
	 */
	@SuppressWarnings("unchecked")
	private void createPlatformCommunicationGraphProblem(
			ArchitectureExplorationProblemModule archExpProblemModule,
			DecoderDependencyGraph execDepGraph) throws CycleFoundException {
		Class<PlatformCommunicationGraphEncoding> pmEnodingClass =
				PlatformCommunicationGraphEncoding.class;
		PlatformCommunicationGraphCreator pmCreator =
				new PlatformCommunicationGraphCreator(execDepGraph,
						(Collection<IExecutionUnitAdapter<?>>)(Collection<?>)systemModelAdapter
								.getDeploymentTargets(),
						(Collection<IMemoryUnitAdapter<?>>)(Collection<?>)systemModelAdapter
								.getMemoryUnits(),
						(Collection<IGatewayUnitAdapter<?>>)(Collection<?>)systemModelAdapter
								.getGatewayUnits(),
						(Collection<ITransmissionUnitAdapter<?>>)(Collection<?>)systemModelAdapter
								.getTransmissionUnits(), systemModelAdapter.getResouceConnections());

		@SuppressWarnings("rawtypes") List<ComposableDecoder> pmDecoders = new ArrayList<>();
		PartitionMappingDecoderGraph pmgDecoder = new PartitionMappingDecoderGraph();
		pmDecoders.add(pmgDecoder);
		execDepGraph.addCreatedEncoding(pmCreator.getCreatedEncodingType());
		archExpProblemModule.registerSubProblem(pmEnodingClass, pmCreator, pmDecoders);
	}

	/**
	 * Creates a task mapping sub-problem (see {@link AbstractTaskMappingEncoding}) which describes
	 * the problem of mapping a logical/functional application architecture to a defined platform
	 * architecture. Note that logical components may also be empty containers in this sub problem
	 * description and which must be instantiated first to derive a valid deployment.
	 * 
	 * @param tmFactory
	 *            Factory which manages the creation tasks associated with task mapping problems
	 * @param archExpProblemModule
	 *            The {@link CompositeProblemModuleBase} declaring the architectural exploration
	 *            problem.
	 * @param dse
	 *            The design space exploration description.
	 * @param execDepGraph
	 *            Dependency graph defining the execution order.
	 * @throws Exception
	 *             if (one of) the sub problem(s) cannot be registered.
	 */
	@SuppressWarnings("unchecked")
	private <S extends InstantiatedTaskMappingEntry, T extends InstantiatedTaskMappingEncoding<S>>
			void createAbstractTaskMappingProblem(TaskMappingFactory<S, T> tmFactory,
					ArchitectureExplorationProblemModule archExpProblemModule,
					DseSpecification dse, DecoderDependencyGraph execDepGraph) throws Exception {
		Class<AbstractTaskMappingEncoding> tmEnodingClass = AbstractTaskMappingEncoding.class;
		AbstractTaskMappingCreator tmCreator =
				tmFactory.createAbstractTaskMappingCreator(execDepGraph);
		@SuppressWarnings("rawtypes") List<ComposableDecoder> abstrTaskDecoders = new ArrayList<>();
		AbstractTaskMappingDecoder<?, ?> atmDecoder = tmFactory.createAbstractTaskMappingDecoder();
		AbstractTaskMappingIdentityDecoder idtDecoder = new AbstractTaskMappingIdentityDecoder();
		abstrTaskDecoders.add(atmDecoder);
		abstrTaskDecoders.add(idtDecoder);
		// TODO: replace with the registration method that uses lists.
		tmCreator.registerAssocDecoders(atmDecoder);
		tmCreator.registerAssocDecoders(idtDecoder);

		execDepGraph.addCreatedEncoding(tmCreator.getCreatedEncodingType());
		archExpProblemModule.registerSubProblem(tmEnodingClass, tmCreator, abstrTaskDecoders);
	}

	/**
	 * Creates a task mapping sub-problem (see {@link InstantiatedTaskMappingEncoding}) which
	 * describes
	 * the problem of instantiating a mapped logical/functional application architecture to a
	 * defined platform architecture. Here, the focus is on optimizing the instantiation of
	 * "abstract" {@link ITaskAdapter}s.
	 * 
	 * @param tmFactory
	 *            Factory which manages the creation tasks associated with task mapping problems
	 * @param archExpProblemModule
	 *            The {@link CompositeProblemModuleBase} declaring the architectural exploration
	 *            problem.
	 * @param dse
	 *            The design space exploration description.
	 * @param execDepGraph
	 *            Dependency graph defining the execution order.
	 * @throws Exception
	 *             if (one of) the sub problem(s) cannot be registered.
	 */
	@SuppressWarnings("unchecked")
	private <S extends InstantiatedTaskMappingEntry, T extends InstantiatedTaskMappingEncoding<S>>
			void createTaskMappingInstantiationProblem(TaskMappingFactory<S, T> tmFactory,
					ArchitectureExplorationProblemModule archExpProblemModule,
					DseSpecification dse, DecoderDependencyGraph execDepGraph) throws Exception {
		// These suppress warnings are required since type erasure in Java does not allow to pass
		// XY<Z>.class.
		StrictTTDecoder<S, T> tmDecoder = tmFactory.createStrictTTDecoder(dse);
		InstantiatedTaskMappingDecoder<S, ?> tgDecoder =
				tmFactory.createInstantiatedTaskMappingDecoder();
		InstantiatedTaskMappingDecoderAcyclic<S, ?> acycTgDecoder =
				tmFactory.createAbstractToAcyclicTaskGraphDecoder();
		@SuppressWarnings("rawtypes") MessageDecoder msgDecoder = tmFactory.createMessageDecoder();
		@SuppressWarnings("rawtypes") TaskMappingCreator tmCreator =
				tmFactory.createTaskMappingCreator(execDepGraph);
		@SuppressWarnings("rawtypes") List<ComposableDecoder> instTaskDecoders = new ArrayList<>();
		instTaskDecoders.add(tgDecoder);
		instTaskDecoders.add(acycTgDecoder);
		instTaskDecoders.add(msgDecoder);
		instTaskDecoders.add(tmDecoder);
		tmCreator.registerAssocDecoders(instTaskDecoders);

		execDepGraph.addCreatedEncoding(tmCreator.getCreatedEncodingType());
		archExpProblemModule.registerSubProblem(InstantiatedTaskMappingEncoding.class, tmCreator,
				instTaskDecoders);
	}

	/**
	 * Creates a new {@link ArchitectureExplorationCreator} which manages the creation of the
	 * concrete architecture exploration problem, i.e. the {@link ArchitectureExplorationCreator}
	 * calls the creators of the genotypes composing the architecture exploration problem.
	 */
	public ArchitectureExplorationCreator createArchitectureExplorationCreator() {
		return new ArchitectureExplorationCreator();
	}

	/**
	 * Returns the {@link TaskMappingFactory} that fits the given {@link ExecutionModel} from the
	 * DSE.
	 * 
	 * @throws UnsupportedDataTypeException
	 *             if the given {@link ExecutionModel} is not "wrapped" to factory in this method.
	 */
	@SuppressWarnings("unchecked")
	public <S extends TaskMappingEntry<?, Partition>>
			TaskMappingFactory<?, ? extends TaskMappingEncoding<?, S>> createTaskMappingFactory(
					ExecutionModel execModel) throws UnsupportedDataTypeException {
		// TODO: Remove the error-prone casting. Remove the differentiation between the failure
		// modes here as well?
		switch(execModel) {
			case TT_FAIL_SILENCE:
				return (TaskMappingFactory<?, ? extends TaskMappingEncoding<?, S>>)FailSilentExecModelFactory
						.getInstance();
			case TT_FAULT_DETECTION_VOTING:
				return (TaskMappingFactory<?, ? extends TaskMappingEncoding<?, S>>)FaultDetectionVotingExecModelFactory
						.getInstance();
			default:
				throw new UnsupportedDataTypeException(
						"Cannot create a factory for the execution model " + execModel.getName() +
								". There is no no factory registered for this executio model.");
		}
	}

	/**
	 * Creates a decoder for the architectural exploration problem that shall be solved by the
	 * DSE.
	 * It delegates the subproblems to sub decoders, e.g. a decoder for the task mapping problem.
	 */
	public ArchitectureDecoder createArchitectureDecoder(DecoderDependencyGraph execdepGraph) {
		return new ArchitectureDecoder(execdepGraph);
	}

	/**
	 * Creates an evaluator for the architectural exploration problem that shall be solved by the
	 * DSE.
	 * It delegates the evaluation of the overall problem to sub-decoders, e.g. an evaluator of the
	 * power consumption of a TT-schedule.
	 */
	public ArchitectureEvaluator createArchitectureEvaluator() {
		return new ArchitectureEvaluator();
	}
}
