/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskgraph.AbstractTaskGraphEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskgraph.TaskGraphEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.genotype.ComposableGenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.exception.ExplorationException;
import org.opt4j.core.Genotype;

/**
 * {@link Genotype} encoding a {@link TaskMappingEncoding} whose
 * {@link InstantiatedTaskMappingEntry}s
 * are instantiated during the construction. Therefore, it requires an
 * {@link AbstractTaskMappingEncoding}.
 * 
 * @author diewald
 */
public abstract class InstantiatedTaskMappingEncoding<T extends InstantiatedTaskMappingEntry>
		extends TaskMappingEncoding<ITaskAdapter<?>, T> {

	/**
	 * Constructor that is initialized with a reference to the DSE specification and the input
	 * system model.
	 */
	protected InstantiatedTaskMappingEncoding(PartitionMappingEncoding partEnc,
			AbstractTaskGraphEncoding<?, ?> tgEncoding) {
		super(partEnc, tgEncoding);
	}

	/** Copy Constructor. */
	protected InstantiatedTaskMappingEncoding(PartitionMappingEncoding partEnc,
			TaskGraphEncoding<?, ?, ?> tgEncoding) {
		super(partEnc, tgEncoding);
	}

	// TODO: Check whether the two constructors below are actually needed.
	/**
	 * Constructor that optionally initializes the encoding to a random encoding
	 * 
	 * @throws ExplorationException
	 *             if no valid allocation targets can be identified for one {@link ITaskAdapter}.
	 */
	public InstantiatedTaskMappingEncoding(PartitionMappingEncoding partEnc,
			AbstractTaskGraphEncoding<?, ?> tgEncoding, boolean initialize)
			throws ExplorationException {
		super(partEnc, tgEncoding, initialize);
	}

	/**
	 * Constructor which derives the resulting {@link InstantiatedTaskMappingEncoding} from the
	 * given {@link AbstractTaskMappingEncoding}.
	 */
	public InstantiatedTaskMappingEncoding(PartitionMappingEncoding partEnc,
			AbstractTaskGraphEncoding<?, ?> tgEncoding, AbstractTaskMappingEncoding abstractEncoding) {
		super(partEnc, tgEncoding);
		this.constraintMap = abstractEncoding.copyConstraintMap(null);
		instantiateAbstractEntries(abstractEncoding);
	}

	/**
	 * Instantiates the {@link ITaskAdapter}s of the given {@link AbstractTaskMappingEncoding} which
	 * must be replaced by a "concrete" {@link ITaskAdapter} representing one possible
	 * implementation of the {@link ITaskAdapter} that shall be replaced. (ref. Diversity)
	 */
	public void instantiateAbstractEntries(AbstractTaskMappingEncoding abstractEncoding) {
		for(AbstractTaskMappingEntry abstractEntry : abstractEncoding.getMappingEntries()) {
			T instantiatedEntry = createEntry(abstractEntry);
			instantiatedEntry.addAssociatedElement(abstractEntry);
			abstractEntry.addAssociatedElement(instantiatedEntry);
			addAllocation(instantiatedEntry);
		}
	}

	/** {@inheritDoc} */
	@Override
	public Collection<IExecutionUnitAdapter<?>> getAllowedAllocations(
			ITaskAdapter<?> deployableComponent) {
		if(deployableComponent.isInstantiation()) {
			deployableComponent = deployableComponent.getReplacedComponent();
		}
		return super.getAllowedAllocations(deployableComponent);
	}

	/** {@inheritDoc} */
	@Override
	public Collection<IExecutionUnitAdapter<?>> getDisAllowedAllocations(
			ITaskAdapter<?> deployableComponent) {
		if(deployableComponent.isInstantiation()) {
			deployableComponent = deployableComponent.getReplacedComponent();
		}
		return super.getDisAllowedAllocations(deployableComponent);
	}

	/** {@inheritDoc} */
	@Override
	public Class<? extends ComposableGenotype> getGTypeToRegister() {
		return InstantiatedTaskMappingEncoding.class;
	}

	/** {@inheritDoc} */
	@Override
	public Class<? extends Phenotype> getPTypeToRegister() {
		return InstantiatedTaskMappingEncoding.class;
	}
}
