/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FailSilentTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.taskmapping.instantiate.FailSilentAbstractTaskMappingDecoder;
import org.opt4j.core.Genotype;
import org.opt4j.core.problem.Decoder;
import org.opt4j.core.start.Opt4JTask;

import com.google.inject.AbstractModule;
import com.google.inject.Inject;
import com.google.inject.Injector;
import com.google.inject.Provides;

/**
 * {@link Decoder} that handles dependencies between {@link Decoder}s, {@link Genotype}s, and
 * {@link Phenotype}s using the Guice framework.
 * 
 * @author diewald
 */
public class GuiceDecoder extends AbstractModule implements Decoder<Genotype, Phenotype> {

	/** Resulting phenotype from the decoding phase that is composed of sub-{@link Phenotype}s. */
	protected Phenotype composedPhenotype;

	/** Defines the desired exploration result type. */
	// TODO: Remove this hard coded line and get the desired result from the backend interface.
	protected Class<? extends Phenotype> resultType = FailSilentTaskMappingEncoding.class;

	/** Injector created by the {@link Opt4JTask}. */
	@Inject
	protected Injector opt4JInjector;

	/** Child injector for the dependency management of the decode phase. */
	protected Injector decodingInjector;

	/** Reference to the input {@link Genotype}. */
	protected Genotype genotype;

	// TODO: Switch from field injections to Constructor injection as far as possible.
	// /** Constructor. */
	// @Inject
	// public GuiceDecoder(Injector opt4JInjector) {
	// decoderInjector = opt4JInjector;
	// }

	/** {@inheritDoc} */
	@Override
	@Provides
	public Phenotype decode(Genotype genotype) {
		// The decode operations are responsible to add the results of the decode operation to the
		// parent phenotype.
		this.genotype = genotype;
		// TODO(AD): See configure(): We need a service machenism and user configuration to select
		// the desired set of activated decoders.
		decodingInjector = opt4JInjector.createChildInjector(this);
		decodingInjector.getInstance(resultType);
		return composedPhenotype;
	}

	/** Provides the composite {@link Genotype} as required by the Guice framework. */
	@Provides
	public Genotype getGenotype() {
		return genotype;
	}

	/** {@inheritDoc} */
	@Override
	protected void configure() {
		// TODO(AD): we need to implement a service mechanism in this configure method. (Plugin
		// Style).
		install(new FailSilentAbstractTaskMappingDecoder());
	}
}
