/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.exception.DecodingException;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.FlatPhenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.problem.CompositeProblemModuleBase;
import org.fortiss.af3.exploration.alg.service.IExplorationEncoding;
import org.opt4j.core.Genotype;
import org.opt4j.core.problem.Decoder;

/**
 * {@link Decoder}s for {@link Phenotype}s must implement this interface and be registered
 * as part of a sub problem to the {@link CompositeProblemModuleBase}, such that the sub problem is
 * solved automatically.
 * 
 * @author diewald
 */
public interface PhenotypeDecoder<P extends Phenotype> {

	/**
	 * Decodes a {@link Phenotype} contained in the given {@link FlatPhenotypeMap} and returns it.
	 * 
	 * @param decodedPhenotypes
	 *            Map of already decoded {@link Phenotype}s.
	 * 
	 * @returns The concrete {@link Phenotype} resulting form the decoding.
	 * 
	 * @throws DecodingException
	 *             If decoding the given {@link Genotype} is not possible.
	 */
	// TODO: update documentation.
	// TODO: Pass the phenotype to be decoded as a separate parameter.
	P decodeComposable(FlatPhenotypeMap<Phenotype> decodedPhenotypes) throws DecodingException;

	/**
	 * Updates the given {@link Phenotype} using the information encoded in the given
	 * {@link FlatPhenotypeMap} (which contains the already decoded {@link Phenotype}s) and returns
	 * it.
	 * 
	 * @param outGenotype
	 *            Hybrid phenotype to be updated.
	 * @param decodedPhenotypes
	 *            Map of already decoded {@link Phenotype}s.
	 * 
	 * @returns The concrete {@link Phenotype} resulting form the decoding.
	 * 
	 * @throws DecodingException
	 *             If decoding the given {@link Genotype} is not possible.
	 */
	P updateComposable(P outGenotype, FlatPhenotypeMap<Phenotype> decodedPhenotypes)
			throws DecodingException;

	/**
	 * Returns the collection of {@link Phenotype} types that are required as direct
	 * inputs to this {@link PhenotypeDecoder}.
	 */
	Collection<Class<? extends IExplorationEncoding>> getRequiredPhenotypeTypes();

	/**
	 * Returns the collection of {@link IExplorationEncoding} that may provide optional information
	 * for the decoding of the input {@link Genotype}s but which are not required for a successful
	 * decoding.
	 */
	Collection<Class<? extends IExplorationEncoding>> getOptionalEncodingTypes();

	/** Returns the type of the decoded {@link Phenotype}. */
	Class<? extends Phenotype> getDecodedPhenotypeType();
}
