/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.evaluate;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.FlatPhenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.opt4j.core.Objectives;
import org.opt4j.core.problem.Evaluator;

import com.google.common.collect.ClassToInstanceMap;

/**
 * Common interface of {@link Evaluator}s of {@link Phenotype}s.
 * 
 * @author diewald
 */
public interface ComposableEvaluator<P extends Object> {
	/**
	 * Passes the {@link Phenotype} to the concrete evaluate method of the concrete
	 * {@link ComposableEvaluator} and returns the resulting {@link Objectives}.
	 * <p>
	 * <b>Implementation note:</b> <br>
	 * The body of the method typically looks as follows: <br>
	 * {@code return decode((ConecretePhenotype)phenotype);}
	 * 
	 * @param clazz
	 *            Type of the {@link Phenotype} to be evaluated.
	 * @param phenotype
	 *            The concrete {@link Phenotype} that shall be decoded.
	 * @param phenotypes
	 *            Map of decoded {@link Phenotype}s which may provide additional input.
	 * @param transformedModels
	 *            TODO
	 * @returns The concrete evaluation results in the form of {@link Objectives}.
	 */
	<CP extends Phenotype> Objectives evaluateComposable(Class<CP> clazz, CP phenotype,
			FlatPhenotypeMap<Phenotype> phenotypes, ClassToInstanceMap<EObject> transformedModels);
}
