/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.taskmapping;

import java.util.ArrayList;
import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.ComposableDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.FlatPhenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.ArchExpSubDecoder;

/**
 * Provides a {@link AbstractTaskMappingEncoding} to the set of {@link Phenotype} types.
 * This {@link ComposableDecoder} is an identity operation.
 * 
 * @author diewald
 */
// TODO: We actually need an additional AbstractTaskMappingDecoder, as this one is not an identity
// decoder anymore. We'd need a clearer concept when to use which operators anyways (Dependent on
// the selected features of the DSE).
public class AbstractTaskMappingIdentityDecoder extends
		TaskMappingDecoderBase<AbstractTaskMappingEntry, AbstractTaskMappingEncoding, AbstractTaskMappingEncoding>
		implements ArchExpSubDecoder {
	/** {@inheritDoc} */
	@Override
	public AbstractTaskMappingEncoding decodeComposable(AbstractTaskMappingEncoding genotype,
			FlatPhenotypeMap<Phenotype> decodedPhenotypes) {
		return genotype;
	}

	/** {@inheritDoc} */
	@Override
	public AbstractTaskMappingEncoding updateComposable(AbstractTaskMappingEncoding inGenotype,
			AbstractTaskMappingEncoding outGenotype,
			FlatPhenotypeMap<Phenotype> decodedPhenotypes) {
		PartitionMappingEncoding pmEncoding =
				decodedPhenotypes.getEncoding(PartitionMappingEncoding.class);
		assert (pmEncoding != null);
		inGenotype.updatePartitionMapping(pmEncoding);
		return alignWithResourceModifications(inGenotype, decodedPhenotypes);
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getRequiredTypes() {
		Collection<Class<?>> reqTypes = new ArrayList<>();
		reqTypes.add(AbstractTaskMappingEncoding.class);
		reqTypes.add(PartitionMappingEncoding.class);
		return reqTypes;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getOptionalTypes() {
		Collection<Class<?>> optTypes = new ArrayList<>();
		return optTypes;
	}

	/** {@inheritDoc} */
	@Override
	public Class<AbstractTaskMappingEncoding> getOutputType() {
		return AbstractTaskMappingEncoding.class;
	}

	/** {@inheritDoc} */
	@Override
	public Class<?> getPrimaryInputType() {
		return AbstractTaskMappingEncoding.class;
	}
}
