/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.taskmapping;

import java.util.ArrayList;
import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.Partition;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.ComposableDecoderBase;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.FlatPhenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.exception.ExplorationException;

/**
 * 
 * @author diewald
 */
public abstract class TaskMappingDecoderBase<S extends TaskMappingEntry<ITaskAdapter<?>, Partition>, T extends TaskMappingEncoding<ITaskAdapter<?>, S>, P extends Phenotype>
		extends ComposableDecoderBase<T, P> {

	/**
	 * Reallocates the {@link IResourceAdapter}s if the set of {@link IResourceAdapter}s was changed
	 * by a previous module. This is required to stay consistent.
	 * 
	 * @param taskMapping
	 *            {@link TaskMappingEncoding} to adapt.
	 * @param decodedPhenotypes
	 *            {@link Phenotype} set required to gather the alignment information, .e.g.,
	 *            resource encodings.
	 * @return Aligned {@link TaskMappingEncoding}.
	 */
	protected T alignWithResourceModifications(T taskMapping,
			FlatPhenotypeMap<Phenotype> decodedPhenotypes) {
		PartitionMappingEncoding pmEncoding =
				decodedPhenotypes.getEncoding(PartitionMappingEncoding.class);
		assert (pmEncoding != null);

		Collection<S> relocateEntries = new ArrayList<>();
		Collection<IResourceAdapter<?>> existingPartitions = pmEncoding.getResources();
		for(S mappingEntry : taskMapping.getMappingEntries()) {
			if(!existingPartitions.contains(mappingEntry.getTarget())) {
				relocateEntries.add(mappingEntry);
			}
		}

		for(S mappingEntry : relocateEntries) {
			taskMapping.removeEntry(mappingEntry);
			try {
				taskMapping.createAndAddRandomEntry(mappingEntry.getSource());
			} catch(ExplorationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		return taskMapping;
	}
}
