/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.taskmapping.instantiate;

import static org.fortiss.tooling.common.util.LambdaUtils.filterSet;

import java.util.ArrayList;
import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FailSilentTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FailSilentTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.ComposableDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.exception.DecodingException;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.FlatPhenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;

import com.google.inject.Provides;

/**
 * Decoder for instantiating an {@link AbstractTaskMappingEncoding} to a
 * {@link FailSilentTaskMappingEncoding}.
 * 
 * @author diewald
 */
// FIXME: The type hierarchy and naming of the classes is extremely confusing. FIX ASAP!
public class FailSilentAbstractTaskMappingDecoder extends
		AbstractTaskMappingDecoder<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding>
/* implements Provider<TaskMappingEncoding> */{

	public FailSilentAbstractTaskMappingDecoder() {

	}

	/** {@inheritDoc} */
	@Override
	public void configure() {
		// install(this);
	}

	/** Input task mapping that may contain components which need to be instantiated. */
	// @Inject
	// AbstractTaskMappingEncoding atmEnc;

	/** Input task graph to perform the mapping. */
	// @Inject
	// AbstractTaskGraphEncoding<?, ?> atgEnc;

	/**
	 * Input mapping of partitions to executions units. This step maps to partitions encoded here.
	 * Additionally, information about the phys. executor may be required for smart mapping
	 * decisions.
	 */
	// @Inject
	// PartitionMappingEncoding partEnc;

	/** {@inheritDoc} */
	@Override
	@Provides
	public FailSilentTaskMappingEncoding decodeComposable(AbstractTaskMappingEncoding atmEnc,
			FlatPhenotypeMap<Phenotype> phenotypeTypeMap) {
		return null;
		// return new FailSilentTaskMappingEncoding(partEnc, atgEnc, atmEnc);
	}

	// TODO: Abstract to the AbstractTaskMappingDecoder.
	/**
	 * {@inheritDoc}
	 * 
	 * @throws DecodingException
	 *             see {@link ComposableDecoder}.
	 */
	@Override
	public FailSilentTaskMappingEncoding
			updateComposable(AbstractTaskMappingEncoding inGenotype,
					FailSilentTaskMappingEncoding outGenotype,
					FlatPhenotypeMap<Phenotype> decodedPhenotypes) throws DecodingException {
		outGenotype.replaceConstraints(inGenotype.getConstraints());

		for(ITaskAdapter<?> comp : inGenotype.getRequesters()) {
			Collection<AbstractTaskMappingEntry> abstrEntries =
					inGenotype.getMappingEntriesOf(comp);
			Collection<FailSilentTaskMappingEntry> instEntries;
			if(comp.isInstantiationRequired()) {
				instEntries =
						filterSet(outGenotype.getMappingEntries(), e -> e.getSource()
								.getReplacedComponent() == comp);
			} else {
				instEntries = outGenotype.getMappingEntriesOf(comp);
			}

			// Missing Associations between instantiated and abstract entries indicate the need to
			// create or remove corresp. entries.
			for(AbstractTaskMappingEntry abstrEntry : abstrEntries) {
				if(abstrEntry.getAssociatedElement().isEmpty()) {
					outGenotype.createAndAddEntry(abstrEntry);
				}
			}

			Collection<FailSilentTaskMappingEntry> rmEntries = new ArrayList<>();
			for(FailSilentTaskMappingEntry instEntry : instEntries) {
				if(instEntry.getAssociatedElement().isEmpty()) {
					rmEntries.add(instEntry);
				}
			}
			rmEntries.forEach(e -> outGenotype.removeEntry(e));
		}

		// Remove all tasks that are present in the output encoding but not in the input encoding.
		Collection<ITaskAdapter<?>> rmTasks = new ArrayList<>();
		for(ITaskAdapter<?> comp : outGenotype.getRequesters()) {
			ITaskAdapter<?> rmComponent = comp;
			if(comp.isInstantiation()) {
				comp = comp.getReplacedComponent();
			}
			if(!inGenotype.getRequesters().contains(comp)) {
				rmTasks.add(rmComponent);
			}
		}
		rmTasks.forEach(e -> outGenotype.removeTask(e));

		for(FailSilentTaskMappingEntry entry : outGenotype.getMappingEntries()) {
			entry.resetTarget(entry.getAssociatedElement(AbstractTaskMappingEntry.class)
					.getTarget());
		}

		return outGenotype;
	}

	/** {@inheritDoc} */
	@Override
	public Class<?> getPrimaryInputType() {
		return AbstractTaskMappingEncoding.class;
	}

	// TODO: The list of parameters injected into the decodeComposable call should be 0, use fields
	// for now.
	// /** {@inheritDoc} */
	// @Override
	// public TaskMappingEncoding get() {
	// return decodeComposable(null, null);
	// }
}
