/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.modeltransformation.deployment;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.component.model.Component;
import org.fortiss.af3.component.model.ComponentArchitecture;
import org.fortiss.af3.deployment.model.Deployment;
import org.fortiss.af3.exploration.alg.dse.modeltransformation.ITransformationModule;
import org.fortiss.af3.exploration.alg.dse.modeltransformation.base.TransformationModuleBase;
import org.fortiss.af3.exploration.alg.dse.sysmodel.mapping.IMappingEntry;
import org.fortiss.af3.exploration.alg.exception.TransformationModuleException;
import org.fortiss.af3.exploration.alg.guava.SubClassToInstanceMap;
import org.fortiss.af3.exploration.alg.service.ExplorationTransformationInputs;
import org.fortiss.af3.exploration.alg.service.IExplorationTransformationService.TransformationContext;
import org.fortiss.af3.platform.model.PlatformArchitecture;

/**
 * Replicates the entries of the deployment parameter table such that every {@link IMappingEntry}
 * indicating the replication of a {@link Component} also has a counterpart describing its
 * deployment specific parameters in the table.
 * 
 * @author diewald
 */
public class DeploymentReplicator extends TransformationModuleBase<Deployment> {

	/** {@inheritDoc} */
	@Override
	public Class<Deployment> getOutputType() {
		return Deployment.class;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getRequiredTypes() {
		Collection<Class<?>> inputTypes = new HashSet<>();
		inputTypes.add(ComponentArchitecture.class);
		inputTypes.add(PlatformArchitecture.class);
		return inputTypes;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getOptionalTypes() {
		return Collections.emptySet();
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<? extends ITransformationModule<?>>> getParentModules() {
		Collection<Class<? extends ITransformationModule<?>>> parentModules = new HashSet<>();
		parentModules.add(ExplorationSolutionToDeployment.class);
		return parentModules;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isOptional() {
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public TransformationState getActivationStage() {
		return TransformationState.REPLICATED;
	}

	/** {@inheritDoc} */
	@Override
	public <N extends EObject, S extends EObject> Deployment transform(TransformationState state,
			TransformationContext context, ExplorationTransformationInputs inputs,
			SubClassToInstanceMap<N> transformedModels, EObject outputModel)
			throws TransformationModuleException {
		// FIXME: Currently broken functionality, as the input set does not contain full
		// architecture models.
		ComponentArchitecture originalCA = null;// inputs.getEncoding(ComponentArchitecture.class);
		if(originalCA == null) {
			throw new TransformationModuleException(this,
					"Could not retrieve the original component architecture from the set of base models. Be sure to add it to the required input types.");
		}

		// FIXME: Currently broken functionality, as the input set does not contain full
		// architecture models.
		PlatformArchitecture originalPA = null;// inputs.getEncoding(PlatformArchitecture.class);
		if(originalPA == null) {
			throw new TransformationModuleException(this,
					"Could not retrieve the original platform architecture from the set of base models. Be sure to add it to the required input types.");
		}

		return (Deployment)outputModel;
	}
}
